/*
 * "Elektricke obvody"
 * Autor: Marian Varga, aug-2000
 */

/*
Pouzivame aritmetiku v poli Z2, t.j. 
iba s cislami z mnoziny {0, 1} a s operaciami
+ def. ako log. xor (+ modulo 2)
* def. ako log. and (* modulo 2)
Pri takejto definicii ide o pole, cize plati
vacsina pravidiel znamych pre pole celych cisel Z 
alebo realnych cisel R, a teda mozeme pouzit aj
zname metody riesenia sustav lin. rovnic.

Potrebujeme vlastne vyriesit sustavu linearnych
rovnic Ax = b, kde
  A je matica (n x m) ovladania vypinacov 
     (1 v stlpci j a riadku i = vypinac j ovlada svetlo i)
  x je stlpcovy m-rozmerny vektor - riesenie
  b je stlpcovy n-rozmerny vektor urcujuci, ktore
     svetla treba zazat (negacia po zlozkach momentalne
     svietiacich svetiel).

Pouzijeme Gaussovu eliminacnu metodu - maticu A upravime
na trojuholnikovu. Potom postupne dosadzujeme od poslednej
neznamej, az kym nenajdeme vhodne riesenie alebo nezistime,
ze riesenie neexistuje. Ak po eliminacii urcita premenna
ma koeficient 0, je "volna" - mozeme si jej hodnotu vybrat.

Lexikalne najmensie riesenie hladame takto:
1. premenne do stlpcov umiestnujeme v opacnom poradi, aby
sme najprv urcovali hodnotu prvej premennej (vypinaca).
2. ak je to mozne, volime pre premennu hodnotu 1, pretoze
tym dostaneme do riesenia vypinac s co najnizsim cislom.
3. po kazdom dosadeni testujeme, ci doteraz dosadene 1
nestacia uz na riesenie (ak vypinace s vacsimi cislami
ostanu na 0), aby sme dosiahli co najkratsie riesenie.

Casova zlozitost je O(m^2 * n) (Gaussova eliminacia).

Optimalizacia by bola mozna pouzitim bitovych operacii
na operacie s vektormi nul a jedniciek.
*/

 
#include <stdio.h>
#define MAX 50

FILE *f, *ff;
int k,n,m,i,j,ii,jj,b,l;
int a[MAX][MAX+1], *aa[MAX], x[MAX];

void pm(void)
{
    int i, j;
    
    printf("matica:\n");
    for (i=0; i<n; i++) {
        for (j=0; j<m; j++)
            printf("%d ", aa[i][j]);

        printf("| %d\n", aa[i][m]);
    }
}

main(void)
{
    /*
    f=fopen("OBVODY.IN", "r");
    ff=fopen("OBVODY.OUT", "w");
    if (!f || !ff) return 1;
    */
    f=stdin; ff=stdout;
    fscanf(f, "%d", &k);
    while (k--)
    {
        /* nacitaj n, m, b, A */
        fscanf(f, "%d%d", &n, &m);
        for (i=0; i<n; i++)
        {
            fscanf(f, "%d", &b);
            a[i][m]=!b;
        }
        for (j=m-1; j>=0; j--)
        {  /* stlpce v opacnom poradi */
            ii=0;
            fscanf(f, "%d", &i); 
            while (i) {
                i--;
                while (ii<i) a[ii++][j]=0;
                a[ii++][j]=1;
                fscanf(f, "%d", &i);
            }
            while (ii<n) a[ii++][j]=0;
        }

        /* inicializuj permutaciu riadkov */
        for (i=0; i<n; i++) aa[i]=a[i];
        /* pm(); */

        /* Ries Ax = b */
        /* Urob trojuh. maticu - GEM */
        for (j=0; j<m; j++)
        { /* eliminuj stlpec j */
            /* hladaj 1 */
            for (i=j; i<n; i++)
                if (aa[i][j]) break;
            if (i<n)
            { /* nasla sa 1 v riadku i */
                int *tmp = aa[i];
                aa[i]=aa[j];
                aa[j]=tmp;
                /* 1 je na aa[j][j] */
                /* zrusme ine 1 v stlpci */
                for (i=j+1; i<n; i++)
                    if (aa[i][j])
                    {
                        for (jj=0; jj<=m; jj++)
                            aa[i][jj]^=aa[j][jj];
                    }
            }
            /* pm(); */
        }

        /* hladaj lex. najmensie riesenie */
        l=0; /* dlzka riesenia */
        for (j=m-1; ; j--)
        { /* nie je uz hotove riesenie? */
            for (i=0; i<n; i++)
                if (aa[i][m]) break;
            /* printf("chk0 i=%d j=%d\n", i, j); */
            if (i==n) break;
            if (j<0)
            {
              /* po dosadeni vsetkych premennych
                 sme nevynulovali pravu stranu
                 => nema riesenie */
              l=-1; break;
            }
            /* dosad 1 */
            if (j<n && aa[j][j])
            { /* pevna hodnota urcena pravou stranou */
		/* printf("pevna\n"); */
                if (aa[j][m])
                { /* tato suradnica je 1 */
                    x[l++]=m-j; /* stlpce mame v opac. poradi */
                    for (i=0; i<n; i++)
                        aa[i][m]^=aa[i][j];
                }
            }
            else
            { /* volna premenna */
		/* printf("volna\n"); */
                if (j<n && aa[j][m])
                { /* 0x = 1 nema riesenie */
			/* printf("0=1\n"); */
                    l=-1; break;
                }
                x[l++]=m-j;
                for (i=0; i<n; i++)
                    aa[i][m]^=aa[i][j];
            }
            /* pm(); */
            /* printf("l=%d ", l); 
            for (ii=0; ii<l; ii++) printf("%d ", x[ii]);
            printf("\n"); */
        }
        
        /* vypis riesenie */
        if (l>=0)
        {
            for (j=0; j<l; j++)
                fprintf(ff, "%d ", x[j]);
            fprintf(ff, "0\n");
        }
        else
            /*fprintf(ff, "Rozsvietit sa neda!\n");*/
            fprintf(ff, "Impossible\n");
    } /* while (k */
    /*fclose(f);
    fclose(ff);*/
    return 0;
}
