#include <stdio.h>


/*************************************************
 problem 5 : 1992 Duke Internet Programming Contest
 written by: Owen Astrachan

 uses dummy rows as first and last (0th and (n+1)st to
 avoid need for range checking.  "Real" data stored in
 rows 1 ... n, cols 0 ... n-1
*************************************************/


#define MAXD 102    /* 2 more than maximimum dimensions */
#define min(x,y)    ((x) < (y) ? (x) : (y))
#define cost(x,y)   memory[(x)][(y)].cost
    
int grid[MAXD][MAXD];
int numRows,numCols;

typedef struct s{
    int cost,lastMove;
} TableStructType;


TableStructType memory[MAXD][MAXD];

static void SolveGrid(int, int);
static void PrintPath(int,int);
static int MinEntry(int,int);


int main()
{
    char buf[MAXD]; /* any "large" number works here */
    int j,k;

    while (fgets(buf,MAXD,stdin) != NULL){
	sscanf(buf,"%d %d",&numRows,&numCols);

	/* read matrix, make room for nth row as row zero */
	for(j=1;j<=numRows;j++){
	    for(k=0;k<numCols;k++){
		scanf("%d",&(grid[j][k]));
	    }
	}
	/* clean up newline */
	fgets(buf,MAXD,stdin);	


	/* copy nth row to zero and 1st row to n+1 */
	for(j=0;j<numCols;j++){
	    grid[0][j] = grid[numRows][j];
	    grid[numRows+1][j] = grid[1][j];
	}

	SolveGrid(numRows,numCols);
    }

    exit (0);
}


static void
SolveGrid(int numRows, int numCols)
{
    int j,k;
    int min;

    

    /* initialize first col of memory to cost of first col of grid */

    for(j=1;j<=numRows;j++){
	memory[j][numCols - 1].cost = grid[j][numCols - 1];
	memory[j][numCols - 1].lastMove = j;
    }
    memory[0][numCols - 1] = memory[numRows][numCols - 1];
    memory[numRows+1][numCols - 1] = memory[1][numCols - 1];

    /* make each col in memory */

    for(k=numCols-2;k>=0;k--){
	for(j=1;j<=numRows;j++){
	    min = MinEntry(j,k);
	    memory[j][k].cost = grid[j][k] + memory[min][k+1].cost;
	    memory[j][k].lastMove = min;
	}
	memory[0][k] = memory[numRows][k];
	memory[numRows+1][k] = memory[1][k];
    }

    /* find start of min path */
    
    min = 1;
    for(j=2;j<=numRows;j++){
	if (memory[j][0].cost < memory[min][0].cost){
	    min = j;
	}
    }
    PrintPath(min,numCols);
    printf("%d\n",memory[min][0].cost);
}


static void
PrintPath(int row,int numCols)
{
    int j;
    for(j=0;j<numCols;j++){
        if(j != 0) printf(" ");
	printf("%d",row);
	row = memory[row][j].lastMove;
    }
    printf("\n");
}


static int
MinEntry(int row,int col)
{
    if (cost(row,col+1) < cost(row-1,col+1)){
	if (cost(row,col+1) < cost(row+1,col+1)){
	    return row;
	}
	else if (cost(row,col+1) == cost(row+1,col+1)){
	    if (row+1 == numRows+1)
		return 1;
	    return row;
	}
	return (row + 1 == numRows+1 ? 1 : row+1);
    }

    if (cost(row,col+1) == cost(row-1,col+1)){
	if (cost(row+1,col+1) < cost(row,col+1)){
	    return (row + 1 == numRows+1 ? 1 : row+1);
	}
	else if (cost(row+1,col+1) == cost(row,col+1)){
	    return (row+1 == numRows+1 ? 1 : row - 1 == 0 ? row:row-1);
	}
	return (row - 1 == 0 ? row : row - 1);
    }

    if (cost(row-1,col+1) < cost(row+1,col+1)){
	return (row-1 == 0 ? numRows : row - 1);
    }
    else if (cost(row+1,col+1) < cost(row-1,col+1)){
	return (row+1 == numRows + 1 ? 1 : row + 1);
    }
    return (row+1 == numRows + 1 ? 1 : row - 1 == 0 ? row + 1 : row - 1);
}
